@@ -240,7 +240,7 @@ class TestTableOrient:
 
     def test_read_json_from_to_json_results(self):
         # GH32383
-        df = pd.DataFrame(
+        df = DataFrame(
             {
                 "_id": {"row_0": 0},
                 "category": {"row_0": "Goods"},
@@ -252,10 +252,13 @@ class TestTableOrient:
             }
         )
         result1 = pd.read_json(df.to_json())
-        result2 = pd.DataFrame.from_dict(json.loads(df.to_json()))
+        result2 = DataFrame.from_dict(json.loads(df.to_json()))
         tm.assert_frame_equal(result1, df)
         tm.assert_frame_equal(result2, df)
 
+    @pytest.mark.filterwarnings(
+        "ignore:an integer is required (got type float)*:DeprecationWarning"
+    )
     def test_to_json(self):
         df = self.df.copy()
         df.index.name = "idx"
@@ -432,6 +435,9 @@ class TestTableOrient:
 
         assert result == expected
 
+    @pytest.mark.filterwarnings(
+        "ignore:an integer is required (got type float)*:DeprecationWarning"
+    )
     def test_date_format_raises(self):
         with pytest.raises(ValueError):
             self.df.to_json(orient="table", date_format="epoch")
@@ -610,13 +616,13 @@ class TestTableOrient:
     )
     def test_warns_non_roundtrippable_names(self, idx):
         # GH 19130
-        df = pd.DataFrame(index=idx)
+        df = DataFrame(index=idx)
         df.index.name = "index"
         with tm.assert_produces_warning():
             set_default_names(df)
 
     def test_timestamp_in_columns(self):
-        df = pd.DataFrame(
+        df = DataFrame(
             [[1, 2]], columns=[pd.Timestamp("2016"), pd.Timedelta(10, unit="s")]
         )
         result = df.to_json(orient="table")
@@ -628,8 +634,8 @@ class TestTableOrient:
         "case",
         [
             pd.Series([1], index=pd.Index([1], name="a"), name="a"),
-            pd.DataFrame({"A": [1]}, index=pd.Index([1], name="A")),
-            pd.DataFrame(
+            DataFrame({"A": [1]}, index=pd.Index([1], name="A")),
+            DataFrame(
                 {"A": [1]},
                 index=pd.MultiIndex.from_arrays([["a"], [1]], names=["A", "a"]),
             ),
@@ -641,7 +647,7 @@ class TestTableOrient:
 
     def test_mi_falsey_name(self):
         # GH 16203
-        df = pd.DataFrame(
+        df = DataFrame(
             np.random.randn(4, 4),
             index=pd.MultiIndex.from_product([("A", "B"), ("a", "b")]),
         )
@@ -670,6 +676,11 @@ class TestTableOrientReader:
             {"floats": [1.0, 2.0, 3.0, 4.0]},
             {"floats": [1.1, 2.2, 3.3, 4.4]},
             {"bools": [True, False, False, True]},
+            {
+                "timezones": pd.date_range(
+                    "2016-01-01", freq="d", periods=4, tz="US/Central"
+                )  # added in # GH 35973
+            },
         ],
     )
     @pytest.mark.skipif(sys.version_info[:3] == (3, 7, 0), reason="GH-35309")
@@ -682,14 +693,7 @@ class TestTableOrientReader:
     @pytest.mark.parametrize("index_nm", [None, "idx", "index"])
     @pytest.mark.parametrize(
         "vals",
-        [
-            {"timedeltas": pd.timedelta_range("1H", periods=4, freq="T")},
-            {
-                "timezones": pd.date_range(
-                    "2016-01-01", freq="d", periods=4, tz="US/Central"
-                )
-            },
-        ],
+        [{"timedeltas": pd.timedelta_range("1H", periods=4, freq="T")}],
     )
     def test_read_json_table_orient_raises(self, index_nm, vals, recwarn):
         df = DataFrame(vals, index=pd.Index(range(4), name=index_nm))
@@ -697,6 +701,53 @@ class TestTableOrientReader:
         with pytest.raises(NotImplementedError, match="can not yet read "):
             pd.read_json(out, orient="table")
 
+    @pytest.mark.parametrize(
+        "idx",
+        [
+            pd.Index(range(4)),
+            pd.Index(
+                pd.date_range(
+                    "2020-08-30",
+                    freq="d",
+                    periods=4,
+                ),
+                freq=None,
+            ),
+            pd.Index(
+                pd.date_range("2020-08-30", freq="d", periods=4, tz="US/Central"),
+                freq=None,
+            ),
+            pd.MultiIndex.from_product(
+                [
+                    pd.date_range("2020-08-30", freq="d", periods=2, tz="US/Central"),
+                    ["x", "y"],
+                ],
+            ),
+        ],
+    )
+    @pytest.mark.parametrize(
+        "vals",
+        [
+            {"floats": [1.1, 2.2, 3.3, 4.4]},
+            {"dates": pd.date_range("2020-08-30", freq="d", periods=4)},
+            {
+                "timezones": pd.date_range(
+                    "2020-08-30", freq="d", periods=4, tz="Europe/London"
+                )
+            },
+        ],
+    )
+    @pytest.mark.skipif(sys.version_info[:3] == (3, 7, 0), reason="GH-35309")
+    def test_read_json_table_timezones_orient(self, idx, vals, recwarn):
+        # GH 35973
+        df = DataFrame(vals, index=idx)
+        out = df.to_json(orient="table")
+        result = pd.read_json(out, orient="table")
+        tm.assert_frame_equal(df, result)
+
+    @pytest.mark.filterwarnings(
+        "ignore:an integer is required (got type float)*:DeprecationWarning"
+    )
     def test_comprehensive(self):
         df = DataFrame(
             {
@@ -724,7 +775,7 @@ class TestTableOrientReader:
     )
     def test_multiindex(self, index_names):
         # GH 18912
-        df = pd.DataFrame(
+        df = DataFrame(
             [["Arr", "alpha", [1, 2, 3, 4]], ["Bee", "Beta", [10, 20, 30, 40]]],
             index=[["A", "B"], ["Null", "Eins"]],
             columns=["Aussprache", "Griechisch", "Args"],
@@ -736,7 +787,7 @@ class TestTableOrientReader:
 
     def test_empty_frame_roundtrip(self):
         # GH 21287
-        df = pd.DataFrame(columns=["a", "b", "c"])
+        df = DataFrame(columns=["a", "b", "c"])
         expected = df.copy()
         out = df.to_json(orient="table")
         result = pd.read_json(out, orient="table")
