@@ -1,15 +1,11 @@
-from collections import OrderedDict
-
 import numpy as np
-from numpy import nan
-from numpy.random import randn
 import pytest
 
 import pandas as pd
-from pandas import DataFrame, Index, MultiIndex, Series
+from pandas import DataFrame, Index, MultiIndex, Series, Timestamp
+import pandas._testing as tm
 from pandas.core.reshape.concat import concat
 from pandas.core.reshape.merge import merge
-import pandas.util.testing as tm
 
 
 @pytest.fixture
@@ -40,13 +36,13 @@ def right():
 @pytest.fixture
 def left_multi():
     return DataFrame(
-        dict(
-            Origin=["A", "A", "B", "B", "C"],
-            Destination=["A", "B", "A", "C", "A"],
-            Period=["AM", "AM", "IP", "AM", "OP"],
-            TripPurp=["hbw", "nhb", "hbo", "nhb", "hbw"],
-            Trips=[1987, 3647, 2470, 4296, 4444],
-        ),
+        {
+            "Origin": ["A", "A", "B", "B", "C"],
+            "Destination": ["A", "B", "A", "C", "A"],
+            "Period": ["AM", "AM", "IP", "AM", "OP"],
+            "TripPurp": ["hbw", "nhb", "hbo", "nhb", "hbw"],
+            "Trips": [1987, 3647, 2470, 4296, 4444],
+        },
         columns=["Origin", "Destination", "Period", "TripPurp", "Trips"],
     ).set_index(["Origin", "Destination", "Period", "TripPurp"])
 
@@ -54,13 +50,13 @@ def left_multi():
 @pytest.fixture
 def right_multi():
     return DataFrame(
-        dict(
-            Origin=["A", "A", "B", "B", "C", "C", "E"],
-            Destination=["A", "B", "A", "B", "A", "B", "F"],
-            Period=["AM", "AM", "IP", "AM", "OP", "IP", "AM"],
-            LinkType=["a", "b", "c", "b", "a", "b", "a"],
-            Distance=[100, 80, 90, 80, 75, 35, 55],
-        ),
+        {
+            "Origin": ["A", "A", "B", "B", "C", "C", "E"],
+            "Destination": ["A", "B", "A", "B", "A", "B", "F"],
+            "Period": ["AM", "AM", "IP", "AM", "OP", "IP", "AM"],
+            "LinkType": ["a", "b", "c", "b", "a", "b", "a"],
+            "Distance": [100, 80, 90, 80, 75, 35, 55],
+        },
         columns=["Origin", "Destination", "Period", "LinkType", "Distance"],
     ).set_index(["Origin", "Destination", "Period", "LinkType"])
 
@@ -196,6 +192,25 @@ class TestMergeMulti:
 
         tm.assert_frame_equal(merged_left_right, merge_right_left)
 
+    def test_merge_multiple_cols_with_mixed_cols_index(self):
+        # GH29522
+        s = pd.Series(
+            range(6),
+            MultiIndex.from_product([["A", "B"], [1, 2, 3]], names=["lev1", "lev2"]),
+            name="Amount",
+        )
+        df = DataFrame({"lev1": list("AAABBB"), "lev2": [1, 2, 3, 1, 2, 3], "col": 0})
+        result = pd.merge(df, s.reset_index(), on=["lev1", "lev2"])
+        expected = DataFrame(
+            {
+                "lev1": list("AAABBB"),
+                "lev2": [1, 2, 3, 1, 2, 3],
+                "col": [0] * 6,
+                "Amount": range(6),
+            }
+        )
+        tm.assert_frame_equal(result, expected)
+
     def test_compress_group_combinations(self):
 
         # ~ 40000000 possible unique groups
@@ -311,11 +326,11 @@ class TestMergeMulti:
             [
                 ["X", "Y", "C", "a", 6],
                 ["X", "Y", "C", "a", 9],
-                ["W", "Y", "C", "e", nan],
+                ["W", "Y", "C", "e", np.nan],
                 ["V", "Q", "A", "h", -3],
                 ["V", "R", "D", "i", 2],
                 ["V", "R", "D", "i", -1],
-                ["X", "Y", "D", "b", nan],
+                ["X", "Y", "D", "b", np.nan],
                 ["X", "Y", "A", "c", 1],
                 ["X", "Y", "A", "c", 4],
                 ["W", "Q", "B", "f", 3],
@@ -365,10 +380,10 @@ class TestMergeMulti:
                 ["c", 0, "x"],
                 ["c", 0, "r"],
                 ["c", 0, "s"],
-                ["b", 1, nan],
+                ["b", 1, np.nan],
                 ["a", 2, "v"],
                 ["a", 2, "z"],
-                ["b", 3, nan],
+                ["b", 3, np.nan],
             ],
             columns=["tag", "val", "char"],
             index=[2, 2, 2, 2, 0, 1, 1, 3],
@@ -390,10 +405,10 @@ class TestMergeMulti:
         left = DataFrame(
             {
                 "id": list("abcde"),
-                "v1": randn(5),
-                "v2": randn(5),
+                "v1": np.random.randn(5),
+                "v2": np.random.randn(5),
                 "dummy": list("abcde"),
-                "v3": randn(5),
+                "v3": np.random.randn(5),
             },
             columns=["id", "v1", "v2", "dummy", "v3"],
         )
@@ -454,38 +469,81 @@ class TestMergeMulti:
         if klass is not None:
             on_vector = klass(on_vector)
 
-        expected = DataFrame(
-            OrderedDict([("a", [1, 2, 3]), ("key_1", [2016, 2017, 2018])])
-        )
+        expected = DataFrame({"a": [1, 2, 3], "key_1": [2016, 2017, 2018]})
 
         result = df.merge(df, on=["a", on_vector], how="inner")
         tm.assert_frame_equal(result, expected)
 
         expected = DataFrame(
-            OrderedDict(
-                [("key_0", [2016, 2017, 2018]), ("a_x", [1, 2, 3]), ("a_y", [1, 2, 3])]
-            )
+            {"key_0": [2016, 2017, 2018], "a_x": [1, 2, 3], "a_y": [1, 2, 3]}
         )
 
         result = df.merge(df, on=[df.index.year], how="inner")
         tm.assert_frame_equal(result, expected)
 
+    @pytest.mark.parametrize("merge_type", ["left", "right"])
+    def test_merge_datetime_multi_index_empty_df(self, merge_type):
+        # see gh-36895
+
+        left = DataFrame(
+            data={
+                "data": [1.5, 1.5],
+            },
+            index=MultiIndex.from_tuples(
+                [[Timestamp("1950-01-01"), "A"], [Timestamp("1950-01-02"), "B"]],
+                names=["date", "panel"],
+            ),
+        )
+
+        right = DataFrame(
+            index=MultiIndex.from_tuples([], names=["date", "panel"]), columns=["state"]
+        )
+
+        expected_index = MultiIndex.from_tuples(
+            [[Timestamp("1950-01-01"), "A"], [Timestamp("1950-01-02"), "B"]],
+            names=["date", "panel"],
+        )
+
+        if merge_type == "left":
+            expected = DataFrame(
+                data={
+                    "data": [1.5, 1.5],
+                    "state": [None, None],
+                },
+                index=expected_index,
+            )
+            results_merge = left.merge(right, how="left", on=["date", "panel"])
+            results_join = left.join(right, how="left")
+        else:
+            expected = DataFrame(
+                data={
+                    "state": [None, None],
+                    "data": [1.5, 1.5],
+                },
+                index=expected_index,
+            )
+            results_merge = right.merge(left, how="right", on=["date", "panel"])
+            results_join = right.join(left, how="right")
+
+        tm.assert_frame_equal(results_merge, expected)
+        tm.assert_frame_equal(results_join, expected)
+
     def test_join_multi_levels(self):
 
         # GH 3662
         # merge multi-levels
         household = DataFrame(
-            dict(
-                household_id=[1, 2, 3],
-                male=[0, 1, 0],
-                wealth=[196087.3, 316478.7, 294750],
-            ),
+            {
+                "household_id": [1, 2, 3],
+                "male": [0, 1, 0],
+                "wealth": [196087.3, 316478.7, 294750],
+            },
             columns=["household_id", "male", "wealth"],
         ).set_index("household_id")
         portfolio = DataFrame(
-            dict(
-                household_id=[1, 2, 2, 3, 3, 3, 4],
-                asset_id=[
+            {
+                "household_id": [1, 2, 2, 3, 3, 3, 4],
+                "asset_id": [
                     "nl0000301109",
                     "nl0000289783",
                     "gb00b03mlx29",
@@ -494,7 +552,7 @@ class TestMergeMulti:
                     "nl0000289965",
                     np.nan,
                 ],
-                name=[
+                "name": [
                     "ABN Amro",
                     "Robeco",
                     "Royal Dutch Shell",
@@ -503,17 +561,24 @@ class TestMergeMulti:
                     "Postbank BioTech Fonds",
                     np.nan,
                 ],
-                share=[1.0, 0.4, 0.6, 0.15, 0.6, 0.25, 1.0],
-            ),
+                "share": [1.0, 0.4, 0.6, 0.15, 0.6, 0.25, 1.0],
+            },
             columns=["household_id", "asset_id", "name", "share"],
         ).set_index(["household_id", "asset_id"])
         result = household.join(portfolio, how="inner")
         expected = (
             DataFrame(
-                dict(
-                    male=[0, 1, 1, 0, 0, 0],
-                    wealth=[196087.3, 316478.7, 316478.7, 294750.0, 294750.0, 294750.0],
-                    name=[
+                {
+                    "male": [0, 1, 1, 0, 0, 0],
+                    "wealth": [
+                        196087.3,
+                        316478.7,
+                        316478.7,
+                        294750.0,
+                        294750.0,
+                        294750.0,
+                    ],
+                    "name": [
                         "ABN Amro",
                         "Robeco",
                         "Royal Dutch Shell",
@@ -521,9 +586,9 @@ class TestMergeMulti:
                         "AAB Eastern Europe Equity Fund",
                         "Postbank BioTech Fonds",
                     ],
-                    share=[1.00, 0.40, 0.60, 0.15, 0.60, 0.25],
-                    household_id=[1, 2, 2, 3, 3, 3],
-                    asset_id=[
+                    "share": [1.00, 0.40, 0.60, 0.15, 0.60, 0.25],
+                    "household_id": [1, 2, 2, 3, 3, 3],
+                    "asset_id": [
                         "nl0000301109",
                         "nl0000289783",
                         "gb00b03mlx29",
@@ -531,7 +596,7 @@ class TestMergeMulti:
                         "lu0197800237",
                         "nl0000289965",
                     ],
-                )
+                }
             )
             .set_index(["household_id", "asset_id"])
             .reindex(columns=["male", "wealth", "name", "share"])
@@ -553,7 +618,7 @@ class TestMergeMulti:
                 expected,
                 (
                     DataFrame(
-                        dict(share=[1.00]),
+                        {"share": [1.00]},
                         index=MultiIndex.from_tuples(
                             [(4, np.nan)], names=["household_id", "asset_id"]
                         ),
@@ -568,13 +633,15 @@ class TestMergeMulti:
         # invalid cases
         household.index.name = "foo"
 
-        with pytest.raises(ValueError):
+        with pytest.raises(
+            ValueError, match="cannot join with no overlapping index names"
+        ):
             household.join(portfolio, how="inner")
 
         portfolio2 = portfolio.copy()
         portfolio2.index.set_names(["household_id", "foo"])
 
-        with pytest.raises(ValueError):
+        with pytest.raises(ValueError, match="columns overlap but no suffix specified"):
             portfolio2.join(portfolio, how="inner")
 
     def test_join_multi_levels2(self):
@@ -582,9 +649,9 @@ class TestMergeMulti:
         # some more advanced merges
         # GH6360
         household = DataFrame(
-            dict(
-                household_id=[1, 2, 2, 3, 3, 3, 4],
-                asset_id=[
+            {
+                "household_id": [1, 2, 2, 3, 3, 3, 4],
+                "asset_id": [
                     "nl0000301109",
                     "nl0000301109",
                     "gb00b03mlx29",
@@ -593,30 +660,36 @@ class TestMergeMulti:
                     "nl0000289965",
                     np.nan,
                 ],
-                share=[1.0, 0.4, 0.6, 0.15, 0.6, 0.25, 1.0],
-            ),
+                "share": [1.0, 0.4, 0.6, 0.15, 0.6, 0.25, 1.0],
+            },
             columns=["household_id", "asset_id", "share"],
         ).set_index(["household_id", "asset_id"])
 
         log_return = DataFrame(
-            dict(
-                asset_id=[
+            {
+                "asset_id": [
                     "gb00b03mlx29",
                     "gb00b03mlx29",
                     "gb00b03mlx29",
                     "lu0197800237",
                     "lu0197800237",
                 ],
-                t=[233, 234, 235, 180, 181],
-                log_return=[0.09604978, -0.06524096, 0.03532373, 0.03025441, 0.036997],
-            )
+                "t": [233, 234, 235, 180, 181],
+                "log_return": [
+                    0.09604978,
+                    -0.06524096,
+                    0.03532373,
+                    0.03025441,
+                    0.036997,
+                ],
+            }
         ).set_index(["asset_id", "t"])
 
         expected = (
             DataFrame(
-                dict(
-                    household_id=[2, 2, 2, 3, 3, 3, 3, 3],
-                    asset_id=[
+                {
+                    "household_id": [2, 2, 2, 3, 3, 3, 3, 3],
+                    "asset_id": [
                         "gb00b03mlx29",
                         "gb00b03mlx29",
                         "gb00b03mlx29",
@@ -626,9 +699,9 @@ class TestMergeMulti:
                         "lu0197800237",
                         "lu0197800237",
                     ],
-                    t=[233, 234, 235, 233, 234, 235, 180, 181],
-                    share=[0.6, 0.6, 0.6, 0.15, 0.15, 0.15, 0.6, 0.6],
-                    log_return=[
+                    "t": [233, 234, 235, 233, 234, 235, 180, 181],
+                    "share": [0.6, 0.6, 0.6, 0.15, 0.15, 0.15, 0.6, 0.6],
+                    "log_return": [
                         0.09604978,
                         -0.06524096,
                         0.03532373,
@@ -638,7 +711,7 @@ class TestMergeMulti:
                         0.03025441,
                         0.036997,
                     ],
-                )
+                }
             )
             .set_index(["household_id", "asset_id", "t"])
             .reindex(columns=["share", "log_return"])
@@ -655,9 +728,9 @@ class TestMergeMulti:
 
         expected = (
             DataFrame(
-                dict(
-                    household_id=[1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 4],
-                    asset_id=[
+                {
+                    "household_id": [1, 2, 2, 2, 2, 3, 3, 3, 3, 3, 3, 4],
+                    "asset_id": [
                         "nl0000301109",
                         "nl0000301109",
                         "gb00b03mlx29",
@@ -671,8 +744,21 @@ class TestMergeMulti:
                         "nl0000289965",
                         None,
                     ],
-                    t=[None, None, 233, 234, 235, 233, 234, 235, 180, 181, None, None],
-                    share=[
+                    "t": [
+                        None,
+                        None,
+                        233,
+                        234,
+                        235,
+                        233,
+                        234,
+                        235,
+                        180,
+                        181,
+                        None,
+                        None,
+                    ],
+                    "share": [
                         1.0,
                         0.4,
                         0.6,
@@ -686,7 +772,7 @@ class TestMergeMulti:
                         0.25,
                         1.0,
                     ],
-                    log_return=[
+                    "log_return": [
                         None,
                         None,
                         0.09604978,
@@ -700,7 +786,7 @@ class TestMergeMulti:
                         None,
                         None,
                     ],
-                )
+                }
             )
             .set_index(["household_id", "asset_id", "t"])
             .reindex(columns=["share", "log_return"])
@@ -768,36 +854,32 @@ class TestJoinMultiMulti:
         if box is not None:
             on_vector = box(on_vector)
 
-        expected = DataFrame(
-            OrderedDict([("a", [1, 2, 3]), ("key_1", [2016, 2017, 2018])])
-        )
+        expected = DataFrame({"a": [1, 2, 3], "key_1": [2016, 2017, 2018]})
 
         result = df.merge(df, on=["a", on_vector], how="inner")
         tm.assert_frame_equal(result, expected)
 
         expected = DataFrame(
-            OrderedDict(
-                [("key_0", [2016, 2017, 2018]), ("a_x", [1, 2, 3]), ("a_y", [1, 2, 3])]
-            )
+            {"key_0": [2016, 2017, 2018], "a_x": [1, 2, 3], "a_y": [1, 2, 3]}
         )
 
         result = df.merge(df, on=[df.index.year], how="inner")
         tm.assert_frame_equal(result, expected)
 
     def test_single_common_level(self):
-        index_left = pd.MultiIndex.from_tuples(
+        index_left = MultiIndex.from_tuples(
             [("K0", "X0"), ("K0", "X1"), ("K1", "X2")], names=["key", "X"]
         )
 
-        left = pd.DataFrame(
+        left = DataFrame(
             {"A": ["A0", "A1", "A2"], "B": ["B0", "B1", "B2"]}, index=index_left
         )
 
-        index_right = pd.MultiIndex.from_tuples(
+        index_right = MultiIndex.from_tuples(
             [("K0", "Y0"), ("K1", "Y1"), ("K2", "Y2"), ("K2", "Y3")], names=["key", "Y"]
         )
 
-        right = pd.DataFrame(
+        right = DataFrame(
             {"C": ["C0", "C1", "C2", "C3"], "D": ["D0", "D1", "D2", "D3"]},
             index=index_right,
         )
@@ -808,3 +890,22 @@ class TestJoinMultiMulti:
         ).set_index(["key", "X", "Y"])
 
         tm.assert_frame_equal(result, expected)
+
+    def test_join_multi_wrong_order(self):
+        # GH 25760
+        # GH 28956
+
+        midx1 = MultiIndex.from_product([[1, 2], [3, 4]], names=["a", "b"])
+        midx3 = MultiIndex.from_tuples([(4, 1), (3, 2), (3, 1)], names=["b", "a"])
+
+        left = DataFrame(index=midx1, data={"x": [10, 20, 30, 40]})
+        right = DataFrame(index=midx3, data={"y": ["foo", "bar", "fing"]})
+
+        result = left.join(right)
+
+        expected = DataFrame(
+            index=midx1,
+            data={"x": [10, 20, 30, 40], "y": ["fing", "foo", "bar", np.nan]},
+        )
+
+        tm.assert_frame_equal(result, expected)
